package be.iminds.ilabt.jfed.lowlevel.api_scripts;

import be.iminds.ilabt.jfed.fedmon.webapi.service.json.Server;
import be.iminds.ilabt.jfed.log.Logger;
import be.iminds.ilabt.jfed.lowlevel.*;
import be.iminds.ilabt.jfed.lowlevel.api.AbstractFederationApi;
import be.iminds.ilabt.jfed.lowlevel.api.FederationMemberAuthorityApi2;
import be.iminds.ilabt.jfed.lowlevel.api.FederationSliceAuthorityApi2;
import be.iminds.ilabt.jfed.lowlevel.api_wrapper.UserAndSliceApiWrapper;
import be.iminds.ilabt.jfed.lowlevel.api_wrapper.impl.AutomaticUserAndSliceApiWrapper;
import be.iminds.ilabt.jfed.lowlevel.connection.JFedConnectionProvider;
import be.iminds.ilabt.jfed.lowlevel.connection.SfaConnection;
import be.iminds.ilabt.jfed.preferences.JFedPreferences;
import be.iminds.ilabt.jfed.util.GeniUrn;
import com.fasterxml.jackson.core.JsonProcessingException;
import io.dropwizard.jackson.Jackson;
import org.slf4j.LoggerFactory;

import javax.annotation.Nonnull;
import javax.inject.Inject;
import java.util.ArrayList;
import java.util.List;

/**
 * GetAllUsers
 * <p/>
 * A script to fetch all users
 */
public class GetAllUsers {
    private static final org.slf4j.Logger LOG = LoggerFactory.getLogger(GetAllUsers.class);

    @Nonnull private final Logger logger;

    @Nonnull private final UserAndSliceApiWrapper samaWrapper;

    @Nonnull private final JFedConnectionProvider connectionProvider;
    @Nonnull private final FederationMemberAuthorityApi2 ma;
    @Nonnull private final FederationSliceAuthorityApi2 sa;

    @Nonnull private final GeniUserProvider geniUserProvider;

    @Inject
    public GetAllUsers(@Nonnull Logger logger,
                       @Nonnull AutomaticUserAndSliceApiWrapper.AutomaticUserAndSliceApiWrapperFactory automaticUserAndSliceApiWrapperFactory,
                       @Nonnull GeniUserProvider geniUserProvider,
                       @Nonnull JFedPreferences jFedPreferences,
                       @Nonnull JFedConnectionProvider connectionProvider) {
        this.logger = logger;
        this.geniUserProvider = geniUserProvider;
        this.connectionProvider = connectionProvider;

        if ((!geniUserProvider.isUserLoggedIn())) {
            throw new IllegalArgumentException("GetProjectsForUsers requires a logged in user");
        }

        Server userServer = geniUserProvider.getLoggedInGeniUser().getUserAuthorityServer();
        if (userServer == null) {
            throw new IllegalArgumentException("GetProjectsForUsers requires a user with an auth server");
        }

        if (userServer.getServices() == null)
            throw new IllegalArgumentException("Cannot initialize AutomaticUserAndSliceApiWrapper: userAuth.getServices() = null (userAuth.urn=" + userServer.getDefaultComponentManagerUrn() + ")");

        if (ApiInfo.hasService(userServer, ApiInfo.ApiName.GENI_CH_SA, 2) &&
                ApiInfo.hasService(userServer, ApiInfo.ApiName.GENI_CH_MA, 2)) {
            sa = new FederationSliceAuthorityApi2(logger, jFedPreferences);
            ma = new FederationMemberAuthorityApi2(logger, jFedPreferences);
        } else {
            throw new IllegalArgumentException("user authority should have a \"Common Federation API\" v2 service");
        }

        this.samaWrapper = automaticUserAndSliceApiWrapperFactory.create();
    }


    private SfaConnection getSAConnection() throws JFedException {
        Server userServer = geniUserProvider.getLoggedInGeniUser().getUserAuthorityServer();
        if (userServer == null) {
            throw new IllegalArgumentException("GetProjectsForUsers requires a user with an auth server");
        }

        return (SfaConnection) connectionProvider.getConnectionByAuthority(
                geniUserProvider.getLoggedInGeniUser(),
                userServer,
                new ApiInfo.Api(ApiInfo.ApiName.GENI_CH_SA, 2));
    }
    private SfaConnection getMAConnection() throws JFedException {
        Server userServer = geniUserProvider.getLoggedInGeniUser().getUserAuthorityServer();
        if (userServer == null) {
            throw new IllegalArgumentException("GetProjectsForUsers requires a user with an auth server");
        }

        return (SfaConnection) connectionProvider.getConnectionByAuthority(
                geniUserProvider.getLoggedInGeniUser(),
                userServer,
                new ApiInfo.Api(ApiInfo.ApiName.GENI_CH_MA, 2));
    }

    @ApiMethod(order = 1, hint = "List all user urns as string")
    public List<String> getAllUsers() throws JFedException {
        samaWrapper.getLocalUserCredentials(logger);
        LOG.info("Got credential for user");

        List<AnyCredential> userCredentials = samaWrapper.getLocalUserCredentials(logger);

        AbstractFederationApi.FederationApiReply<AbstractFederationApi.MemberInfoList> reply =
                ma.lookupMember(getSAConnection(), userCredentials, null, null, null);

        if (!reply.getGeniResponseCode().isSuccess()) {
            throw new JFedException("Error when querying user projects: "+reply.getGeniResponseCode()+" -> "+reply.getRawResult());
        }

        AbstractFederationApi.MemberInfoList memberInfoList = reply.getValue();
        if (memberInfoList == null) {
            throw new JFedException("No value in reply from SA");
        }

        List<String> memberUrnStrings = new ArrayList<>();

        for (GeniUrn memberUrn : memberInfoList.keySet()) {
            memberUrnStrings.add(memberUrn.toString());
        }

        LOG.info("Got all users: {}", memberUrnStrings);

        return memberUrnStrings;
    }

    public String getAllUsersAsJson() throws JFedException {
        List<String> r = getAllUsers();
        try {
            return Jackson.newObjectMapper().writerWithDefaultPrettyPrinter().writeValueAsString(r);
        } catch (JsonProcessingException e) {
            throw new JFedException("Error converting result to JSON", e);
        }
    }
}
